/*
 * Config.java, 31 mai 2005
 * 
 * This file is part of xtnd-commons.
 *
 * Copyright © 2005 Johan Cwiklinski
 *
 * File :               Config.java
 * Author's email :     johan@x-tnd.be
 * Author's Website :   http://www.x-tnd.be/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package be.xtnd.commons;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.beans.PropertyVetoException;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.security.NoSuchProviderException;
import java.security.Security;
import java.text.MessageFormat;
import java.util.Iterator;
import java.util.List;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JOptionPane;
import javax.swing.JPasswordField;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.WindowConstants;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.openpgp.PGPEncryptedDataGenerator;
import org.bouncycastle.openpgp.PGPException;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;
import org.jdom.xpath.XPath;

import be.xtnd.commons.gui.EscapeInternalFrame;
import be.xtnd.commons.gui.MainGui;
import be.xtnd.commons.security.Cryptographie;
import be.xtnd.commons.security.EncryptionClass;

import com.jeta.forms.components.panel.FormPanel;
import com.jgoodies.looks.plastic.PlasticInternalFrameUI;

/**
*   La classe Configuration va obtenir et mettre à jour les infos de
*   configuration à partir des fichiers de configuration
*   stockés sous [rep_courant]/config
*   Elle génère également le fichier de config par défaut s'il n'existe pas,
* 	et permet l'affichage d'une fenêtre de configuration.
*
* @since 31/05/2005
* @author Johan CWIKLINKSI
* @version 1.1
*/
public class Config{
	private String xmlPath;
	private JTextField mysqlServer, mysqlUser, mysqlDb;
	private JTextField [] provider, providerName, providerPort, providerAttributes;
	private JComboBox[] providerMode;
	private JRadioButton[] providerDefault;
	private JPasswordField mysqlPassword, softRootPassword;
	/** Nom de la fenêtre */
	public final static String name = "preferences";
	private String sql_provider, sql_server, sql_db, sql_user, sql_password, arguments, mode="",
		root_soft_password, lang, server_name, port, provider_name, crypto_mode = "nomode";
	private List providers;
	private boolean modified=false;
	/** Définit si la configuration est chargée ou non */
	public boolean loaded = false;
	private org.jdom.Document document;
	private Element racine;
	private static final char[] encryptPassPhrase="2]aRF05$ue".toCharArray();
	private Dimension fieldsSize = new Dimension(150,20);
	private JTabbedPane tabbedPane;//container onglets
	private JRadioButton bouncy_crypt, simple_crypt;
	static Logger logger = Logger.getLogger(Config.class.getName());
	
	/**
	 * Construit la configuration de l'application.
	 * 
	 * @param xmlPath nom du fichier xml de configuration (dans le répertoire {application}/.config)
	 */
	public Config(String xmlPath) {
		PropertyConfigurator.configure(ClassLoader.getSystemResource("be/xtnd/commons/log4j.properties"));
		this.xmlPath = ".config/"+xmlPath;
	}
	
	/**
	 * Charge la configuration depuis le fichier XML
	 *  
	 * @param defaultConf <code>boolean</code> permettant de spécifier le chargement
	 * d'un fichier de configuration de défaut. Utile pour la création d'un nouveau fichier
	 * de configuration ou lorsqu'une information important est manquante.
	 * defaultConf à true permet à l'application d'être lancée même si la configuration est
	 * incorrecte. Certaines options nécéssaires seront tout de même chargées.
	 */
	public void loadConfig(boolean defaultConf) {
		logger.debug("*** LOADING PREFERENCES ***");
		Security.addProvider(new BouncyCastleProvider());

        try {                   	
    	    SAXBuilder sxb = new SAXBuilder();
    	    this.document = sxb.build(new File(xmlPath));

    	    this.racine = document.getRootElement();

    		this.providers = racine.getChild("sql_providers").getChildren();
			XPath langPath = XPath.newInstance("//lang[@default='yes']");
			Element langs = (Element) langPath.selectSingleNode(document);
			this.lang = langs.getText()+", file "+langs.getAttributeValue("file")+".jar";

			if(!defaultConf){
	        	XPath providerPath = XPath.newInstance("//sql_provider[@default='yes']");
				Element node = (Element) providerPath.selectSingleNode(document);
				this.provider_name = node.getAttributeValue("name");
				this.sql_provider = node.getText();
				this.port = node.getAttributeValue("port");
				this.server_name = racine.getChild("sql_server").getText();
				this.sql_db = racine.getChild("sql_db").getText();

				this.sql_server = "jdbc:"+provider_name+":";
				mode = node.getAttributeValue("mode");
				if(mode.equalsIgnoreCase("server")){
					this.sql_server += "//";
					if(server_name.length()>0){
						this.sql_server += server_name;
						if(this.port.length()>0) this.sql_server += ":"+this.port;
						this.sql_server += "/";
					}else{
						MainGui.s.end();
						logger.debug("          [Failed]");
						int response = JOptionPane.showConfirmDialog(
								MainGui.desktop, 
								MessagesCommons.getString("window.config.messages.noserver"),
								MessagesCommons.getString("window.config.messages.noserver.title"),
								JOptionPane.YES_NO_OPTION,
								JOptionPane.INFORMATION_MESSAGE);
			    		if (response == JOptionPane.YES_OPTION) {
			    			loadConfig(true);
			    			if(MainGui.verifUnicite(name,null)){
			    				//TODO vérifier si le null ci dessous ne pose pas de problème
			    				new ConfigWindow(null);
			    			}
			    			return;
			    		}
					}
				}
				this.sql_server += sql_db;
				arguments = node.getAttributeValue("arguments");
				if(arguments.length()>0){
					this.sql_server += arguments; 
				}
				
				this.sql_user = racine.getChild("sql_user").getText();
		        
				this.crypto_mode = racine.getChild("cryptomode").getText();
		        
				if(crypto_mode.equalsIgnoreCase("bouncy")){
					byte[] pass_sql = racine.getChild("sql_password").getText().getBytes();
			        this.sql_password = new String(EncryptionClass.decrypt(
			        		pass_sql,
							encryptPassPhrase));
			        byte[] pass_appli = racine.getChild("root_soft_password").getText().getBytes();
					this.root_soft_password = new String(EncryptionClass.decrypt(
			        		pass_appli,
							encryptPassPhrase));
				}else if(crypto_mode.equalsIgnoreCase("simple")){
					this.sql_password = Cryptographie.decrypte(racine.getChild("sql_password").getText());
					this.root_soft_password = Cryptographie.decrypte(racine.getChild("root_soft_password").getText());
				}
			}
			loaded = true;
			logger.debug("          [Ok]");
		} catch (FileNotFoundException e) {
			MainGui.s.end();
			logger.debug("          [Failed]");
			logger.warn("FileNotFoundException while loading configuration from "+xmlPath);
			Object[] args = {xmlPath};
			int response = JOptionPane.showConfirmDialog(
					MainGui.desktop, 
					MessageFormat.format(MessagesCommons.getString("window.config.messages.noexist"), args),
					MessagesCommons.getString("window.config.messages.noexist.title"),
					JOptionPane.YES_NO_OPTION,
					JOptionPane.INFORMATION_MESSAGE);
    		if (response == JOptionPane.YES_OPTION) {
    			makeConfig();
    		}else{
    			JOptionPane.showMessageDialog(
    					MainGui.desktop, 
						MessagesCommons.getString("window.config.messages.nofile"),
						MessagesCommons.getString("window.config.messages.nofile.title"),
						JOptionPane.OK_OPTION+JOptionPane.ERROR_MESSAGE);
    			System.exit(0);
    		}
		} catch (JDOMException e) {
			logger.debug("          [Failed]");
			logger.fatal("JDomException while loading configuration : "+e.getMessage());
		} catch (NoSuchProviderException e1) {
			logger.debug("          [Failed]");
			logger.fatal("NoSuchProviderException while loading configuration : "+e1.getMessage());
		} catch (IOException e1) {
			logger.debug("          [Failed]");
			logger.fatal("IOException while loading configuration : "+e1.getMessage());
		} catch (PGPException e1) {
			logger.debug("          [Failed]");
			logger.fatal("PGPException while loading configuration : "+e1.getMessage());
		}
	}
	 
	/**
	 * Re-créée un fichier de configuration valide à partir
	 * de celui contenu dans le package be.xtnd.conf
	 *
	 */
    private void makeConfig(){
			try {
				org.jdom.Document documentOrig;
				SAXBuilder sxb = new SAXBuilder();
				documentOrig = sxb.build(ClassLoader.getSystemResource("be/xtnd/conf/config-orig.xml"));
	            XMLOutputter sortie = new XMLOutputter();
				sortie.output(documentOrig, new FileOutputStream(xmlPath));
				
    			JOptionPane.showMessageDialog(
    					MainGui.desktop, 
						MessagesCommons.getString("window.config.messages.default"),
						MessagesCommons.getString("window.config.messages.default.title"),
						JOptionPane.OK_OPTION+JOptionPane.INFORMATION_MESSAGE);
    			try {
    				loadConfig(true);
    				//TODO vérifier si le null ci dessous ne pose pas de problème
    				new ConfigWindow(null);
    			} catch (Exception e1) {
					logger.warn(e1.getMessage());
				}
			} catch (JDOMException e) {
				logger.fatal("JDOMException while making config : "+e.getMessage()+" - EXITING");
				System.exit(0);
			} catch (FileNotFoundException e) {
				logger.fatal("FileNotFoundException while making config : "+e.getMessage()+" - EXITING");
				System.exit(0);
			} catch (IOException e) {
				logger.fatal("IOException while making config : "+e.getMessage()+" - EXITING");
				System.exit(0);
			}
    }
    
    /**
     * Sauvegarde la configuration en cours dans le fichier xml
     * 
     * @param close indique que la fenêtre doit être fermée après
     * enregistrement. Cet argument est sans objet dorénavant et sera 
     * supprimé par la suite.
     */
    private void saveConfig(boolean close){
		String dialog_message = new String();
		String dialog_titre = new String();
		String dialog_error = new String();
		String sqlPass=null, softPass=null;
    	boolean result=true;		
		int dialog_type = 0;

		try{
			racine.getChild("sql_server").setText(mysqlServer.getText());
			racine.getChild("sql_db").setText(mysqlDb.getText());
			racine.getChild("sql_user").setText(mysqlUser.getText());

			if(bouncy_crypt.isSelected()){
				sqlPass = 
					new String(EncryptionClass.encrypt(
							String.copyValueOf(mysqlPassword.getPassword()).getBytes(), 
							encryptPassPhrase,
							null,
							PGPEncryptedDataGenerator.AES_256, 
							true)
					);
				softPass = 
					new String(EncryptionClass.encrypt(
							String.copyValueOf(softRootPassword.getPassword()).getBytes(), 
							encryptPassPhrase,
							null,
							PGPEncryptedDataGenerator.AES_256, 
							true)
					);		
			}else if(simple_crypt.isSelected()){
				sqlPass = Cryptographie.crypte(new String(mysqlPassword.getPassword()));
				softPass = Cryptographie.crypte(new String(softRootPassword.getPassword()));
			}
			
			racine.getChild("sql_password").setText(sqlPass);

			racine.getChild("root_soft_password").setText(softPass);

			String crypto = (bouncy_crypt.isSelected())?"bouncy":"simple";
			racine.getChild("cryptomode").setText(crypto);
			
			for(int y =0; y < tabbedPane.getTabCount(); y++){
				try {
					XPath providerPath = XPath.newInstance("//sql_provider[@name='"+tabbedPane.getTitleAt(y)+"']");
					Element node = (Element) providerPath.selectSingleNode(document);
					node.getAttribute("name").setValue(providerName[y].getText());
					node.getAttribute("port").setValue(providerPort[y].getText());
					node.getAttribute("mode").setValue(providerMode[y].getSelectedItem().toString());
					node.getAttribute("arguments").setValue(providerAttributes[y].getText());					
					String strdefault = new String("no");
					if(providerDefault[y].isSelected()) strdefault = "yes";
					node.getAttribute("default").setValue(strdefault);
					node.setText(provider[y].getText());   		
				} catch (JDOMException e2) {e2.printStackTrace();}
	    	} 

            XMLOutputter sortie = new XMLOutputter();
			sortie.output(document, new FileOutputStream(xmlPath));
		} catch (FileNotFoundException e) {
			logger.warn("FileNotFoundException saving config : "+e.getMessage()+" - USER ADVICED");
			dialog_error="erreur : "+e.getMessage();
			result=false;
		} catch (IOException e) {
			logger.warn("IOException saving config : "+e.getMessage()+" - USER ADVICED");
			dialog_error="erreur : "+e.getMessage();
			result=false;
		} catch (NoSuchProviderException e1) {
			logger.warn("NoSuchProviderException saving config : "+e1.getMessage()+" - USER ADVICED");
			dialog_error="erreur : "+e1.getMessage();
			result=false;
		} catch (PGPException e1) {
			logger.warn("PGPException saving config : "+e1.getMessage()+" - USER ADVICED");
			dialog_error="erreur : "+e1.getMessage();
			result=false;
		}

		if(result){
			dialog_message=MessagesCommons.getString("window.config.messages.saved");
			dialog_titre=MessagesCommons.getString("window.config.messages.saved.title");
			dialog_type=JOptionPane.INFORMATION_MESSAGE;
		}else{
			close=false;
			dialog_message=MessagesCommons.getString("window.config.messages.notsaved")+"\n("+dialog_error+")";
			dialog_titre=MessagesCommons.getString("window.config.messages.notsaved");	
			dialog_type=JOptionPane.ERROR_MESSAGE;
		}
		JOptionPane.showMessageDialog(MainGui.desktop, dialog_message,dialog_titre,dialog_type);
		loadConfig(false);
		if(result && close){
			//configFrame.dispose();
			modified=false;
		}  	
    }
    
    /**
     * Ouvre une fenêtre pour modifier/voir/enregistrer
     * la configuration depuis le fichier XML de configuration.
     * 
     * 
     * @author Johan Cwiklinski
     * @version 1.1
     * 
     * @see be.xtnd.commons.gui.EscapeInternalFrame
     * @see Config
     */
    public class ConfigWindow extends EscapeInternalFrame implements ActionListener{
		private static final long serialVersionUID = 8839486849341653324L;
		private JButton okay, annuler, appliquer, simple_crypto_gui;
    	private JLabel serveur_label, db_label, db_user_label, db_passe_label, passe_label, 
			general_label, datas_label, crypt_label;
    	private GuiCommons commons = new GuiCommons();
    	private MainGui parentFrame;
    	
    	/**
    	 * Construit la fenêtre de configuration
    	 * @param parent fenêtre MainGui parente
    	 */
    	public ConfigWindow(MainGui parent){
    		super();
    		this.parentFrame = parent;
    		super.setTitle(MessagesCommons.getString("window.config.title"));
            putClientProperty(
                    PlasticInternalFrameUI.IS_PALETTE,
                    Boolean.TRUE);
    		FormPanel pane = new FormPanel( "be/xtnd/commons/gui/descriptions/preferences.jfrm" );

    		initLabels(pane);
    		
			mysqlServer = pane.getTextField("serveur");
			configFields(mysqlServer, server_name);
			mysqlServer.setCaretPosition(0);


			mysqlPassword = (JPasswordField)pane.getTextField("base_passe");
			configFields(mysqlPassword, sql_password);
			
	    	softRootPassword = (JPasswordField)pane.getTextField("appli_passe");
			configFields(softRootPassword, root_soft_password);

			mysqlUser = pane.getTextField("base_user");
			configFields(mysqlUser, sql_user);
			
			mysqlDb = pane.getTextField("base");
			configFields(mysqlDb, sql_db);
			
			simple_crypt = pane.getRadioButton("simple_crypt");
			simple_crypt.setText(MessagesCommons.getString("window.config.labels.crypto.simple"));
			simple_crypt.setToolTipText(MessagesCommons.getString("window.config.labels.crypto.simple.tooltip"));
			if(crypto_mode.equalsIgnoreCase("simple")) simple_crypt.setSelected(true);
			simple_crypt.addActionListener(this);
			
			bouncy_crypt = pane.getRadioButton("bouncy_crypt");
			bouncy_crypt.setText(MessagesCommons.getString("window.config.labels.crypto.bouncy"));
			bouncy_crypt.setToolTipText(MessagesCommons.getString("window.config.labels.crypto.bouncy.tooltip"));
			if(crypto_mode.equalsIgnoreCase("bouncy")) bouncy_crypt.setSelected(true);
        	bouncy_crypt.addActionListener(this);

			simple_crypto_gui = (JButton)pane.getButton("simple_crypto_gui");
			simple_crypto_gui.setText(MessagesCommons.getString("window.config.labels.simple_crypto_gui"));
			simple_crypto_gui.setToolTipText(MessagesCommons.getString("window.config.labels.simple_crypto_gui.tooltip"));
			simple_crypto_gui.addActionListener(new SimpleClik());
			
    		okay = (JButton)pane.getButton("okay");
    		commons.buildCommonButton(okay, "window.config.buttons.ok", null);
    		okay.addActionListener(new OkClik());
    		
    		appliquer = (JButton)pane.getButton("appliquer");
    		commons.buildCommonButton(appliquer, "window.config.buttons.apply", null);
    		appliquer.addActionListener(new ApplyClik());

    		annuler = (JButton)pane.getButton("annuler");
    		commons.buildCommonButton(annuler, "window.config.buttons.cancel", null);
    		annuler.addActionListener(new CancelClik());
    		
    		buildTabs(pane);
    		
    		add(pane);
    		
    		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
    		setClosable(true);

        	pack();
        	setVisible(true);
        	setName(name);
        	setTitle(MessagesCommons.getString("window.config.title"));
        	getRootPane().setDefaultButton(okay);
    		MainGui.desktop.add(this,JLayeredPane.MODAL_LAYER);
        	setLocation(MainGui.centerOnDesktop(getSize()));
        	try {
    			setSelected(true);
    		} catch (PropertyVetoException e) {}			

    	}

    	/**
    	 * Actions de modification sur la fenêtre de configuration
    	 * @param e
    	 */
        public void actionPerformed(ActionEvent e) {
        	modified = true;
        }
    	
    	/**
    	 * Initialise les différents <code>JLabels</code>
    	 * 
    	 * @param pane <code>FormPanel</code> à utiliser pour récupérer les valeurs
    	 */
    	private void initLabels(FormPanel pane){
    		serveur_label = pane.getLabel("serveur_label");
    		serveur_label.setText(MessagesCommons.getString("window.config.labels.server"));
    		serveur_label.setToolTipText(MessagesCommons.getString("window.config.labels.server.tooltip"));
    		
			db_label = pane.getLabel("base_label");
			db_label.setText(MessagesCommons.getString("window.config.labels.database"));
			db_label.setToolTipText(MessagesCommons.getString("window.config.labels.database.tooltip"));
			
			db_user_label = pane.getLabel("user_label");
			db_user_label.setText(MessagesCommons.getString("window.config.labels.user"));
			db_user_label.setToolTipText(MessagesCommons.getString("window.config.labels.user.tooltip"));
			
			db_passe_label = pane.getLabel("user_passe_label");
			db_passe_label.setText(MessagesCommons.getString("window.config.labels.password"));
			db_passe_label.setToolTipText(MessagesCommons.getString("window.config.labels.password.tooltip"));
			
			passe_label = pane.getLabel("appli_passe_label");
			passe_label.setText(MessagesCommons.getString("window.config.labels.app_password"));
			passe_label.setToolTipText(MessagesCommons.getString("window.config.labels.app_password.tooltip"));
			
			crypt_label = pane.getLabel("cryptage_label");
			crypt_label.setText(MessagesCommons.getString("window.config.labels.crypto"));
			crypt_label.setToolTipText(MessagesCommons.getString("window.config.labels.crypto.tooltip"));
			
			general_label = pane.getLabel("general_label");
			general_label.setText(MessagesCommons.getString("window.config.labels.general"));
			
			datas_label = pane.getLabel("datas_label");
			datas_label.setText(MessagesCommons.getString("window.config.labels.datas"));    		
    	}
    	
    	/**
    	 * Construit un <code>JTabbedPane</code> en fonction
    	 * des préférences des providers sql
    	 * 
    	 * @param pane <code>FormPanel</code> à utiliser pour récupérer le
    	 * <code>JTabbedPane</code>
    	 */
    	private void buildTabs(FormPanel pane){
    		tabbedPane = pane.getTabbedPane("tabs");
    		
    		Iterator i = providers.iterator();
    		int count = 0;
    		int dimTablo = providers.size();
    		
    		providerMode = new JComboBox[dimTablo];
    		provider = new JTextField[dimTablo];
    		providerName = new JTextField[dimTablo];
    		providerPort = new JTextField[dimTablo];
    		providerDefault = new JRadioButton[dimTablo];
    		providerAttributes = new JTextField[dimTablo];
    		
        	ButtonGroup group = new ButtonGroup();

        	while(i.hasNext()){
        		FormPanel tab = new FormPanel( "be/xtnd/commons/gui/descriptions/database_option_tab.jfrm" );

        		JLabel modeLabel = tab.getLabel("mode_label");
        		modeLabel.setText(MessagesCommons.getString("window.config.labels.datas.mode"));
        		modeLabel.setToolTipText(MessagesCommons.getString("window.config.labels.datas.mode.tooltip"));
        		
            	JLabel providerLabel = tab.getLabel("base_provider_label");
            	providerLabel.setText(MessagesCommons.getString("window.config.labels.datas.provider"));
            	providerLabel.setToolTipText(MessagesCommons.getString("window.config.labels.datas.provider.tooltip"));

            	JLabel providerNameLabel = tab.getLabel("base_nom_label");
            	providerNameLabel.setText(MessagesCommons.getString("window.config.labels.datas.name"));
            	providerNameLabel.setToolTipText(MessagesCommons.getString("window.config.labels.datas.name.tooltip"));

            	JLabel providerPortLabel = tab.getLabel("base_port_label");
            	providerPortLabel.setText(MessagesCommons.getString("window.config.labels.datas.port"));
            	providerPortLabel.setToolTipText(MessagesCommons.getString("window.config.labels.datas.port.tooltip"));

            	JLabel providerDefaultLabel = tab.getLabel("base_defaut_label");
            	providerDefaultLabel.setText(MessagesCommons.getString("window.config.labels.datas.default"));
            	providerDefaultLabel.setToolTipText(MessagesCommons.getString("window.config.labels.datas.default.tooltip"));
            	
            	JLabel attributesLabel = tab.getLabel("attributs_label");
            	attributesLabel.setText(MessagesCommons.getString("window.config.labels.datas.attributes"));
            	attributesLabel.setToolTipText(MessagesCommons.getString("window.config.labels.datas.attributes.tooltip"));

        	    Element courant = (Element)i.next();
            	tabbedPane.addTab(courant.getAttributeValue("name"),tab);

            	providerMode[count] = tab.getComboBox("mode");
            	providerMode[count].addItem("server");
            	providerMode[count].addItem("file");
            	providerMode[count].setSelectedItem(courant.getAttributeValue("mode"));
            	
            	provider[count] = tab.getTextField("base_provider");
            	configFields(provider[count], courant.getText());
            	
        		providerName[count] = tab.getTextField("base_nom");
            	configFields(providerName[count], courant.getAttributeValue("name"));
            	
            	providerPort[count] = tab.getTextField("base_port");
            	configFields(providerPort[count], courant.getAttributeValue("port"));
            	
            	providerAttributes[count] = tab.getTextField("attributs");
            	configFields(providerAttributes[count], courant.getAttributeValue("arguments"));
            	
            	providerDefault[count] = tab.getRadioButton("base_defaut");
            	providerDefault[count].addActionListener(this);
                group.add(providerDefault[count]);
            	
                if(courant.getAttribute("default").getValue().equals("yes")){
            		providerDefault[count].setSelected(true);
            		tabbedPane.setSelectedIndex(tabbedPane.getComponentCount()-1);
            	}            	
            	count++;
        	}
    		
    	}
    	
    	/**
         * Configure chaque <code>JTextField</code> et lui associe notamment
         * un <code>KeyListener</code> pour répérer les modifications
         * 
    	 * @param field <code>JTextField</code> à modifier
    	 * @param text texte à afficher
    	 */
    	private void configFields(JTextField field, String text){
        	field.setText(text);
        	field.setPreferredSize(fieldsSize);
        	field.addKeyListener(new KeyAdapter() {
        		public void keyTyped(KeyEvent e) {
        			modified = true;
        			}
    		});
        }
    	
    	/**
         * Configure chaque <code>JPasswordField</code> et lui associe notamment
         * un <code>KeyListener</code> pour répérer les modifications
         * 
    	 * @param field <code>JPasswordField</code> à modifier
    	 * @param passe mot de passe à afficher
    	 */
        private void configFields(JPasswordField field, String passe){
        	field.setText(passe);
        	field.setPreferredSize(fieldsSize);
        	field.addKeyListener(new KeyAdapter() {
        		public void keyTyped(KeyEvent e) {
        			modified = true;
        			}
       			});
        }
        
        /* ACTIONS */
        
    	class SimpleClik implements ActionListener {
    		/** 
    		 * Action du click sur le bouton Simple
    		 * 
    		 *  @param e ActionEvent
    		 */
    		public void actionPerformed(ActionEvent e) {
    			if(MainGui.verifUnicite("cryptowindow",null)){
        			Cryptographie crypto = new Cryptographie();
        			crypto.new CryptoWindow();    				
    			}
    		}
    	}
    	
    	class OkClik implements ActionListener {
    		/**
    		 * Action du click sur le bouton OK. Affiche un message de confirmation 
    		 * si les données ont été modifiées, recharge la base de données 
    		 * puis ferme la fenêtre.
    		 * @param e ActionEvent
    		 */
    		public void actionPerformed(ActionEvent e) {
    			if(modified){
    				int response = JOptionPane.showConfirmDialog(
    						MainGui.desktop, 
							MessagesCommons.getString("window.config.messages.modified"),
							MessagesCommons.getString("window.config.messages.modified.title"),
							JOptionPane.YES_NO_OPTION,
							JOptionPane.QUESTION_MESSAGE);
    				if (response == JOptionPane.YES_OPTION) {
    					saveConfig(true);
    				}
    			}
    			modified=false;
    			try {parentFrame.loadDb();} catch (Exception e1) {
    				//FIXME gestion des éventuelles erreurs
    			}
    			dispose();
    		}
    	}

    	class ApplyClik implements ActionListener {
    		/**
    		 * Action du click sur le boutton appliquer. Enregistre les modifications.
    		 * @param e ActionEvent
    		 */
    		public void actionPerformed(ActionEvent e) {
    			if(modified){saveConfig(false);}
    			modified=false;
    		}
    	}
    	
    	/** Définit l'action du click sur le boutton Annuler */
    	class CancelClik implements ActionListener {
    		/**
    		 * Action du click sur le boutton annuler. Ferme la fenêtre sans enregistrer.
    		 * @param e ActionEvent
    		 */
    		  public void actionPerformed(ActionEvent e) {
    		  	dispose();
    		  }
    	}
    }

	/**
	 * @return Renvoie provider_name.
	 */
	public String getProvider_name() {
		return provider_name;
	}

	/**
	 * @return Renvoie lang.
	 */
	public String getLang() {
		return lang;
	}

	/**
	 * @return Renvoie sql_provider.
	 */
	public String getSql_provider() {
		return sql_provider;
	}

	/**
	 * @return Renvoie sql_server.
	 */
	public String getSql_server() {
		return sql_server;
	}

	/**
	 * @return Renvoie sql_user.
	 */
	public String getSql_user() {
		return sql_user;
	}

	/**
	 * @return Renvoie sql_password.
	 */
	public String getSql_password() {
		return sql_password;
	}

	/**
	 * @return Renvoie mode.
	 */
	public String getMode() {
		return mode;
	}

	/**
	 * @return Renvoie sql_db.
	 */
	public String getSql_db() {
		return sql_db;
	}

	/**
	 * @return Renvoie server_name.
	 */
	public String getServer_name() {
		return server_name;
	}

	/**
	 * @return Renvoie port.
	 */
	public String getPort() {
		return port;
	}

	/**
	 * @return Renvoie arguments.
	 */
	public String getArguments() {
		return arguments;
	}
}
