/*
 * GuiCommons.java, 14 juin 2005
 * 
 * This file is part of xtnd-commons.
 *
 * Copyright © 2005 Johan Cwiklinski
 *
 * File :               GuiCommons.java
 * Author's email :     johan@x-tnd.be
 * Author's Website :   http://www.x-tnd.be/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package be.xtnd.commons;

import java.awt.BorderLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.ResourceBundle;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultStyledDocument;

import be.xtnd.commons.gui.MainGui;

import com.jeta.forms.components.panel.FormPanel;
import com.jgoodies.looks.Options;

/**
 * Utiles communs au GUI
 * Les différents méthodes permettent
 * pour la plupart de peupler des composants
 * (comme les <code>JMenu</code>, <code>JMenuItem</code>,
 * <code>JButton</code>, <code>JComboBox</code>, ...)
 * 
 * @author Johan Cwiklinski
 */
public class GuiCommons {

	private int longueur;
	private static JLabel license, title, url;
	private ResourceBundle bundle;

	/** Constructeur par défaut, 
	 * utlisé par {@link Config.ConfigWindow ConfigWindow} 
	 */
	public GuiCommons(){}
	
	/**
	 * Constructeur par défaut
	 * @param bundle ResourceBundle à lire
	 */
	public GuiCommons(ResourceBundle bundle){
		this.bundle = bundle;
	}
	
	/**
	 * Crée une barre de deux bouttons (Ok et Annuler).
	 * Définit le bouton Ok par défaut, attribue les 
	 * <code>ActionListener</code>
	 * 
	 * @param frame JInternalFrame à laquelle doivent être ajoutés les boutons
	 * @param okayEvent listener du bouton ok
	 */
	public void createButtonBar(JInternalFrame frame, ActionListener okayEvent){
		FormPanel pane = new FormPanel( "be/xtnd/commons/gui/descriptions/buttons_bar.jfrm" );

		JButton okay = (JButton)pane.getButton("okay");
		okay.setText(MessagesCommons.getString("window.buttons.okay"));
		okay.addActionListener(okayEvent);
		frame.getRootPane().setDefaultButton(okay);
		
		JButton cancel = (JButton)pane.getButton("cancel");
		cancel.setText(MessagesCommons.getString("window.buttons.cancel"));
		cancel.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				MainGui.desktop.getSelectedFrame().dispose();
			}
		});
		frame.getContentPane().add(pane,BorderLayout.SOUTH);
	}
		
	/**
	 * Crée un header en fonction du logo passé en paramètre
	 * pour les fenêtres about et le splash screen
	 * 
	 * @param pane le <code>FormPanel</code> à formatter
	 * @param logo le logo à afficher
	 * @param bundle ResourceBundle à lire
	 */
	public static void createHeader(FormPanel pane, String logo, ResourceBundle bundle){
		license = pane.getLabel("license");
		license.setText(bundle.getString("appli.copyright"));
		
		title = pane.getLabel("title");
		title.setText(bundle.getString("appli.name")+" v"+bundle.getString("appli.version"));
		
		url = pane.getLabel("url");
		url.setText(bundle.getString("appli.url"));
		
		JLabel image = pane.getLabel("image");
		image.setIcon(new ImageIcon(ClassLoader.getSystemResource(bundle.getString("appli."+logo)))); //$NON-NLS-1$
	}
	
	/**
	 * Limite la zone au nombre de caractères <code>caracts</code>
	 * 
	 * @param caracts : nombre de caractères de la limite
	 * 
	 * @return doc -> JTextField.setDocument(doc)
	 * 
	 * @see DefaultStyledDocument
	 * 
	 * @deprecated lors d'une utilisation multiple, la valeur est toujours la dernière.
	 * Remplacé par {@link be.xtnd.validators.FieldLengthVerifier#FieldLengthVerifier(int) FieldLengthVerifier}.
	 * 
	 */
	@Deprecated
	public DefaultStyledDocument doc(int caracts){
		this.longueur = caracts;
		DefaultStyledDocument docu = new DefaultStyledDocument(){
			private static final long serialVersionUID = -6870710569372996547L;

			public void insertString(int offs, String str, AttributeSet a)
			throws BadLocationException{
				System.err.println("size : " + longueur);
				if( (getLength() + str.length()) <= longueur ){
					System.err.println("throws correctly exception");
					super.insertString(offs, str, a);
				}else{
					System.err.println("error occured during doc sized");
					Toolkit.getDefaultToolkit().beep();
				}
			}
		};
		return docu;
	}

	/**
	 * Crée un nouveau <code>JComboBox</code>
	 * à l'aide de l'<code>ArrayList</code> passé
	 * en paramètre
	 * 
	 * @param list <code>ArrayList</code> contenant les valeurs
	 * 
	 * @see JComboBox
	 * @see ArrayList
	 * 
	 * @return nouveau <code>JComboBox</code>
	 */
	public JComboBox buildNewCombo(ArrayList list){
		JComboBox combo = new JComboBox();
		combo.addItem(MessagesCommons.getString("window.messages.select"));
		for(int i = 0; i<list.size(); i++){
			combo.addItem(list.get(i));
		}
		return combo;
	}

	/**
	 * Remplit un <code>JComboBox</code> à l'aide
	 * de l'<code>ArrayList</code> passé en 
	 * paramètre
	 * 
	 * @param box <code>JComboBox</code> à peupler
	 * @param list <code>ArrayList</code> contenant les valeurs
	 * 
	 * @see JComboBox
	 * @see ArrayList
	 */
	public void buildCombo(JComboBox box, ArrayList list){
		box.addItem(MessagesCommons.getString("window.messages.select"));
		for(int i = 0; i<list.size(); i++){
			box.addItem(list.get(i));
		}
	}

	/**
	 * Crée une entrée de menu
	 * 
	 * @param menu menu à modifier
	 * @param i18n référence de la locale
	 * @param icon image du menu
	 * 
	 * @see JMenu
	 * 
	 * @return JMenu
	 */
	public JMenu menuEntry(JMenu menu, String i18n, Icon icon){
		System.out.println("  Menu "+bundle.getString(i18n));
		menu.setMnemonic(bundle.getString(i18n+".mnemonic").charAt(0));
		menu.setText(bundle.getString(i18n));
		menu.setIcon(icon);	
		return menu;
	}
	
	/**
	 * Crée une entrée de sous menu
	 * 
	 * @param item item à modifier
	 * @param icon image de l'item
	 * @param i18n référence de la locale
	 * @param keystroke Raccourci clavier
	 * 
	 * @see JMenuItem
	 * 
	 * @return JMenuItem
	 */
	public JMenuItem submenuEntry(JMenuItem item, Icon icon, String i18n, KeyStroke keystroke){
		System.out.println("     Sous menu "+bundle.getString(i18n));
	    item.setToolTipText(bundle.getString(i18n+".tooltip"));
	    item.setIcon(icon);
	    item.setMnemonic(bundle.getString(i18n+".mnemonic").charAt(0));
	    item.setText(bundle.getString(i18n));
	    item.setAccelerator(keystroke);
	    return item;
	}
	
	/**
	 * Crée un sous menu déroulant
	 * 
	 * @param item sous menu
	 * @param icon image du sous menu
	 * @param i18n référence de la locale
	 * 
	 * @see JMenu
	 * 
	 * @return JMenu
	 */
	public JMenu submenuEntry(JMenu item, Icon icon, String i18n){
		System.out.println("     Sous menu "+bundle.getString(i18n)); 
	    item.setIcon(icon);
	    item.setMnemonic(bundle.getString(i18n+".mnemonic").charAt(0));
	    item.setText(bundle.getString(i18n));
	    return item;
	}
	
	/**
	 * Ajuste les propriétés du bouton en paramètre
	 * 
	 * @param button bouton à modifier
	 * @param i18n référence de la locale
	 * @param icon icône associée
	 * 
	 * @see JButton
	 * 
	 * @return JButton
	 */
	public JButton buildButton(JButton button, String i18n, Icon icon){
		button.putClientProperty(Options.IS_NARROW_KEY, Boolean.TRUE);
	  	button.setIcon(icon);
	  	button.setText(bundle.getString(i18n));
	  	button.setToolTipText(bundle.getString(i18n+".tooltip"));
	  	button.setMnemonic(bundle.getString(i18n+".mnemonic").charAt(0));
	  	return button;
	}
		
	/**
	 * Ajuste les propriétés du bouton en paramètre
	 * Utilisé pour les boutons généraux des packages 
	 * be.xtnd.commons.*
	 * 
	 * @param button bouton à modifier
	 * @param i18n référence de la locale
	 * @param icon icône associée
	 * 
	 * @see JButton
	 * 
	 * @return JButton
	 */
	public JButton buildCommonButton(JButton button, String i18n, Icon icon){
		button.putClientProperty(Options.IS_NARROW_KEY, Boolean.TRUE);
	  	button.setIcon(icon);
	  	button.setText(MessagesCommons.getString(i18n));
	  	button.setToolTipText(MessagesCommons.getString(i18n+".tooltip"));
	  	button.setMnemonic(MessagesCommons.getString(i18n+".mnemonic").charAt(0));
	  	return button;		
	}
	
	/**
	 * Ajuste les propriétés du bouton en paramètre
	 * pour affichage sur la toolbar
	 * 
	 * @param button bouton à modifier
	 * @param i18n référence de la locale
	 * @param icon icône associée
	 * 
	 * @see JButton
	 * 
	 * @return JButton
	 */
	public JButton buildBarButton(JButton button, String i18n, Icon icon){
		button.putClientProperty(Options.IS_NARROW_KEY, Boolean.TRUE);
	  	button.setIcon(icon);
	  	button.setToolTipText(bundle.getString(i18n+".tooltip"));
	  	button.setMnemonic(bundle.getString(i18n+".mnemonic").charAt(0));
	  	return button;
	}
}