/*
 * HelpWindow.java, 5 juin 2005
 * 
 * This file is part of xtnd-commons.
 *
 * Copyright © 2005 Johan Cwiklinski
 *
 * File :               HelpWindow.java
 * Author's email :     johan@x-tnd.be
 * Author's Website :   http://www.x-tnd.be/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package be.xtnd.commons;

import java.awt.Dimension;
import java.awt.Point;
import java.beans.PropertyVetoException;
import java.net.URL;
import java.text.MessageFormat;
import java.util.ResourceBundle;

import javax.help.BadIDException;
import javax.help.HelpSet;
import javax.help.HelpSetException;
import javax.help.JHelp;
import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import be.xtnd.commons.gui.EscapeInternalFrame;
import be.xtnd.commons.gui.MainGui;

/**
 * Permet d'afficher une fenêtre d'aide
 * 
 * @author Johan Cwiklinski
 */
public class HelpWindow extends JInternalFrame{
	private static final long serialVersionUID = -4034930522338806800L;
	/** Nom de la fenêtre */
	public static final String name = "aide";
	/** Position de la fenêtre */
	public static Point location;
	static Logger logger = Logger.getLogger(HelpWindow.class.getName());
	private JHelp helpViewer = null; 
	private String help_id;
	
	/**
	 * Affiche un id particulier de l'aide
	 * en lieu et place de la page root
	 * 
	 * @param id id courant de l'aide
	 */
	public void setCurrentId(String id){
		this.help_id = id;
		helpViewer.setCurrentID(id);
	}
	
	/**
	 * Crée une fenêtre d'aide
	 *
	 * @param bundle ResourceBundle à lire
	 */
	public HelpWindow(ResourceBundle bundle){
		PropertyConfigurator.configure(ClassLoader.getSystemResource("be/xtnd/commons/log4j.properties"));
		try {  
			ClassLoader cl = HelpWindow.class.getClassLoader();  
			URL url = HelpSet.findHelpSet(cl, bundle.getString("appli.help_file"));  
			helpViewer = new JHelp(new HelpSet(cl, url));  

			EscapeInternalFrame frame = new EscapeInternalFrame();
			Object[] args = {bundle.getString("appli.name")};
			frame.setTitle(MessageFormat.format(MessagesCommons.getString("window.help.title"), args));

			frame.getContentPane().add(helpViewer);
			Dimension dim = new Dimension(770,500);
			frame.setMinimumSize(dim);
			frame.setPreferredSize(dim);
			frame.setSize(dim);
			frame.pack();
			
			frame.setIconifiable(true);
			frame.setClosable(true);
			frame.setResizable(true);
			frame.setMaximizable(true);
			frame.setName(name);
			frame.setVisible(true);
			
			MainGui.desktop.add(frame);	
			frame.toFront();
			try {
				frame.setSelected(true);
			} catch (PropertyVetoException e) {}
			frame.setLocation(MainGui.centerOnDesktop(frame.getSize()));		
		}catch(HelpSetException e){
			logger.error("HelpSetException : file "+bundle.getString("appli.help_file")+" not found");
			JOptionPane.showMessageDialog(
					MainGui.desktop, 
					MessagesCommons.getString("window.help.inactive_help"),
					MessagesCommons.getString("window.help.inactive_help_title"),
					JOptionPane.OK_OPTION);
		}catch (BadIDException e){
			logger.warn("BadIdException : id "+this.help_id+" not found");
			JOptionPane.showMessageDialog(
					MainGui.desktop, 
					MessagesCommons.getString("window.help.unknow_index"),
					MessagesCommons.getString("window.help.unknow_index_title"), 
					JOptionPane.INFORMATION_MESSAGE + JOptionPane.OK_OPTION);
		}catch (RuntimeException e) {  
			logger.fatal("RuntimeException !!" + e.getMessage());
			JOptionPane.showMessageDialog(MainGui.desktop, 
					MessagesCommons.getString("window.help.system_error"),
					MessagesCommons.getString("window.help.system_error_title"), 
					JOptionPane.WARNING_MESSAGE + JOptionPane.OK_OPTION);
		}catch (Exception e){
			logger.error("Exception : "+e.getMessage());
			JOptionPane.showMessageDialog(
					MainGui.desktop, 
					MessagesCommons.getString("window.help.unknown_error"),
					MessagesCommons.getString("window.help.unknown_error_title"), 
					JOptionPane.WARNING_MESSAGE + JOptionPane.OK_OPTION);
		}
	}
}