/*
 * Print.java, 26 juin 2005
 * 
 * This file is part of xtnd-commons.
 *
 * Copyright © 2005 Johan Cwiklinski
 *
 * File :               Print.java
 * Author's email :     johan@x-tnd.be
 * Author's Website :   http://www.x-tnd.be/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package be.xtnd.commons;

import java.sql.Connection;
import java.sql.ResultSet;
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JRResultSetDataSource;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.design.JRJdtCompiler;
import net.sf.jasperreports.engine.design.JasperDesign;
import net.sf.jasperreports.engine.xml.JRXmlLoader;

import be.xtnd.commons.db.Database;
import be.xtnd.commons.gui.MainGui;
import be.xtnd.commons.gui.PrintPreview;


/**
 * Affiche la fenêtre d'impression JasperReports
 * Le fichier jrxml original doit se trouver dans be/xtnd/conf
 * 
 * @author Johan Cwiklinski
 * @version 1.0
 */
public class Print {
	protected String modelUrl;
	protected ResultSet rs;
	//protected boolean withRs = false;
    protected Map parameters = new HashMap();
    protected JasperDesign design;
	static Logger logger = Logger.getLogger(Print.class.getName());

	/**
	 * Construit un <code>PrintPreview</code> simple.
	 * 
	 * @param url chemin des modèles d'impression
	 * @param model nom du modèle d'impression
	 * 
	 * @see PrintPreview
	 */
	public Print(String url, String model){
		this(url, model, null, null);
		/*this.modelUrl = url+"/"+model;
		print();*/
	}

    /**
     * Construit un <code>PrintPreview</code> basé
     * sur un <code>ResultSet</code>.
     * 
     * @param url chemin des modèles d'impression
     * @param model nom du modèle d'impression
     * @param rs <code>ResultSet</code> : données à imprimer
     * 
     * @see PrintPreview
     */
	public Print(String url, String model, ResultSet rs){
		this(url, model, rs, null);
		/*this.rs = rs;
		this.withRs = true;
		this.modelUrl = url+"/"+model;
		print();*/
	}

	/**
	 * Construit un <code>PrintPreview</code> 
	 * avec un tableau de paramètres <code>HashMaps</code>
	 * 
	 * @param url url du chemin de smodèles d'impression
	 * @param model nom du modèle d'impression
	 * @param params le tableau <code>HashMaps</code> des paramètres
	 * 
	 * @see PrintPreview
	 * @see HashMap
	 */
	public Print(String url, String model, HashMap params){
		this(url, model, null, params);
		/*this.modelUrl = url+"/"+model;
		this.parameters = params;
		print();*/
	}

	/**
	 * Construit un <code>PrintPreview</code> basé 
	 * sur un <code>ResultSet</code>, avec un tableau 
	 * de paramètres <code>HashMaps</code>
	 * 
	 * @param url url du chemin des modèles d'impression
	 * @param model nom du modèle d'impression
	 * @param rs le ResultSet à afficher
	 * @param params le tableau <code>HashMaps</code> des paramètres
	 * 
	 * @see PrintPreview
	 * @see HashMap
	 */
	public Print(String url, String model, ResultSet rs, HashMap params){
		this.rs = rs;
		//this.withRs = true;
		this.modelUrl = url+"/"+model;
		this.parameters = params;
		print();	    
	}

	/**
	 * Construit un <code>PrintPreview</code> simple depuis un
	 * objet JasperDesign
	 * @param design JasperDesign
	 */
	public Print(JasperDesign design){
		this(design, null, null);
	} 

	/**
	 * Construit un <code>PrintPreview</code> basé sur un ResultSet
	 * @param design JasperDesign
	 * @param rs ResultSet à utiliser
	 */
	public Print(JasperDesign design, ResultSet rs){
		this(design, rs, null);
	} 
	
	/**
	 * Construi un <code>PrintPreview</code> avec les 
	 * parmaètres
	 * @param design JasperDesign
	 * @param params Paramètres du report
	 */
	public Print(JasperDesign design, HashMap params){
		this(design, null, params);
	}
	
	/**
	 * Construi un <code>PrintPreview</code> depuis un bojet
	 * JasperDesign basé sur
	 * un ResultSet avec des paramètres
	 * @param design JasperDesign
	 * @param rs ResultSet à utiliser
	 * @param params Paramètres du report
	 */
	public Print(JasperDesign design, ResultSet rs, HashMap params){
		this.rs = rs;
		this.parameters = params;
		this.design = design;
		print();
	} 

	/**
	 * Génère le preview
	 */
	private void print(){
		PropertyConfigurator.configure(ClassLoader.getSystemResource("be/xtnd/commons/log4j.properties"));
		try {
			JasperDesign jasperDesign;
			jasperDesign = (this.design==null)?JRXmlLoader.load(modelUrl):this.design;
			JasperReport jasperReport = new JRJdtCompiler().compileReport(jasperDesign);
	        
			Database database = (MainGui.db != null)?MainGui.db:new Database(true);			
			Connection connection = database.getConnection();
				        
	        JasperPrint jasperPrint = null;
	        if(this.rs!=null){
	        	JRResultSetDataSource jrRS = new JRResultSetDataSource(rs);
	        	jasperPrint = JasperFillManager.fillReport(jasperReport, 
	        			parameters, jrRS);	   
	        }else{
	        	jasperPrint = JasperFillManager.fillReport(jasperReport, 
	        			parameters, connection);
	        }
	        new PrintPreview(jasperPrint);
	        
		} catch (JRException e1) {
			logger.error("JRException when try printing : "+modelUrl+" : "+e1.getMessage());
			//e1.printStackTrace();
		}
		
	}
	
	/* impossible de recopier le fichier de cette façon, le fichier produit est corrompu */
	/*private boolean fileExists(String path){
		File fichier = new File(modelUrl);
		if (!fichier.exists()) {
			System.out.println("          [Failed]");
    			JOptionPane.showMessageDialog(
    					MainGui.desktop, 
    					MessagesCommons.getString("window.print.messages.error"),
    					MessagesCommons.getString("window.print.messages.error.title"),
						JOptionPane.OK_OPTION+JOptionPane.ERROR_MESSAGE);
		}
		return fichier.exists();
	}*/
}
