/*
 * WindowMenu.java, 31 mai 2005
 * 
 * This file is part of xtnd-commons.
 *
 * Copyright © 2005 Johan Cwiklinski
 *
 * File :               WindowMenu.java
 * Author's email :     johan@x-tnd.be
 * Author's Website :   http://www.x-tnd.be/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package be.xtnd.commons;

/**
 * Menu fenetre (réorganisation, liste)
 * 
 * @author Johan Cwiklinski
 */
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyVetoException;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;

import be.xtnd.commons.gui.MainGui;

/**
 * Composant <code>JMenu</code> qui implémente les fonctionnalités
 * d'un menu "Fenêtre" standard pour les application en mode MDI
 * 
 * @see JMenu
 */
public class WindowMenu extends JMenu {
	private static final long serialVersionUID = 8743582773682766512L;
	private MDIDesktopPane desktop;
    private JMenuItem cascade=new JMenuItem(MessagesCommons.getString("menus.window.cascade"));
    private JMenuItem tile=new JMenuItem(MessagesCommons.getString("menus.window.mosaic"));
    private JMenuItem close = new JMenuItem (MessagesCommons.getString("menus.window.close"));

    /**
     * Construit le <code>JMenu</code> "fenêtre".
     * 
     * @param desktop le <code>JDesktopPane</code> auquel
     * les fenêtres sont rattachées.
     */
    public WindowMenu(final MDIDesktopPane desktop) {
        this.desktop=desktop;
        setText(MessagesCommons.getString("menus.window"));
        setMnemonic(MessagesCommons.getString("menus.window.mnemonic").charAt(0));
        cascade.setMnemonic(MessagesCommons.getString("menus.window.cascade.mnemonic").charAt(0));
        cascade.setToolTipText(MessagesCommons.getString("menus.window.cascade.tooltip"));
        cascade.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                WindowMenu.this.desktop.cascadeFrames();
            }
        });
        tile.setMnemonic(MessagesCommons.getString("menus.window.mosaic.mnemonic").charAt(0));
        tile.setToolTipText(MessagesCommons.getString("menus.window.mosaic.tooltip"));
        tile.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                WindowMenu.this.desktop.tileFrames();
            }
        });
        close.setMnemonic(MessagesCommons.getString("menus.window.close.mnemonic").charAt(0));
        close.setToolTipText(MessagesCommons.getString("menus.window.close.tooltip"));
        close.setIcon(MainGui.closeImage);
        close.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                JInternalFrame f = desktop.getSelectedFrame();
                if (f != null) {
                      f.dispose();
                }
            }
        });
        //close.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W,Event.CTRL_MASK));
        
        addMenuListener(new MenuListener() {
            public void menuCanceled (MenuEvent e) {}

            public void menuDeselected (MenuEvent e) {
                removeAll();
            }

            public void menuSelected (MenuEvent e) {
                buildChildMenus();
            }
        });
    }
    
    /**
     * Construit les menus enfants selon l'état courant du bureau
     */
    private void buildChildMenus() {
        int i;
        ChildMenuItem menu;
        JInternalFrame[] array = desktop.getAllFrames();

        add(cascade);
        add(tile);
        addSeparator();
        add(close);
        if (array.length > 0) addSeparator();
        cascade.setEnabled(array.length > 0);
        tile.setEnabled(array.length > 0);

        for (i = 0; i < array.length; i++) {
            menu = new ChildMenuItem(array[i]);
            menu.setState(i == 0);
            menu.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent ae) {
                    JInternalFrame frame = ((ChildMenuItem)ae.getSource()).getFrame();
                    frame.moveToFront();
                    try {
                        frame.setSelected(true);
                    } catch (PropertyVetoException e) {
                        e.printStackTrace();
                    }
                }
            });
            menu.setIcon(array[i].getFrameIcon());
            add(menu);
        }
    }

    /**
     * <code>JCheckBoxMenuItem</code> représentant chacune des
     * fenêtres ouvertes
     */
    class ChildMenuItem extends JCheckBoxMenuItem {
		private static final long serialVersionUID = -3559813875378638729L;
		private JInternalFrame frame;

		/**
		 * 
		 * @param frame JInternalFrame
		 */
        public ChildMenuItem(JInternalFrame frame) {
            super(frame.getTitle());
            this.frame=frame;
        }

        /**
         * 
         * @return JInternalFrame
         */
        public JInternalFrame getFrame() {
            return frame;
        }
    }
}
