/*
 * ResultSetTableModelFactory.java, 31 mai 2005
 * 
 * This file is part of xtnd-commons.
 *
 * Copyright Â© 2005 Johan Cwiklinski
 *
 * File :               ResultSetTableModelFactory.java
 * Author's email :     johan@x-tnd.be
 * Author's Website :   http://www.x-tnd.be/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package be.xtnd.commons.db;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import be.xtnd.commons.db.Database;

/**
 * Crée un objet <code>ResultSetTableModel</code> qui peut être affiché
 * dans un composant Swing <code>JTable</code>
 * 
 * @author Johan Cwiklinski
 * @version 1.0
 **/
public class ResultSetTableModelFactory {
    Database database;
    boolean oldMode = false; //convénience pour l'ancienne méthode, à supprimer
    
    /**
     * Constructeur par défaut
     * @deprecated la construction d'un tel objet devrait toujours se
     * baser sur une connexion existante, afin de limiter le nombre de 
     * connexions établies. Remplacée par {@link #ResultSetTableModelFactory(Database)}
     */
    @Deprecated
    public ResultSetTableModelFactory(){
    	database = new Database();
    	database.setConnectionParameters(
    			ResultSet.TYPE_SCROLL_INSENSITIVE,
				ResultSet.CONCUR_READ_ONLY);
    	oldMode = true;
    }
    
    
    /** Constructeur par défaut
     * Construit un RestultSetTableModel à l'aide de
     * la base de données fournie en paramètre
     * @param db base de données à utiliser   
     *  
     */
    public ResultSetTableModelFactory(Database db){
    	database = db;
    }
    
    /**
     * Envoie une requête à la base de données, les charge dans un
     * <code>ResultSet</code> et retourne un <code>ResultSetTableModel</code>
     * qui permet d'afficher les résultats dans une <code>JTable</code>
     * 
     * @param query la requête à éxécuter
     * @return ResultSetTableModel
     * @throws SQLException
     **/
    public ResultSetTableModel getResultSetTableModel(String query)
        throws SQLException{
    	ResultSet r = null;

    	if(oldMode){//convénience pour l'ancienne méthode, à supprimer
    		database.connect(); 
    		r = database.execQuery(query);
    	}else{
    		Statement stmt = database.getConnection().createStatement(
    				ResultSet.TYPE_SCROLL_INSENSITIVE,
    				ResultSet.CONCUR_READ_ONLY);
    		r = database.execQuery(query, stmt);
    	}
    	return new ResultSetTableModel(r);
    }
}
