/*
 * HsqldbErrors.java, 25 mai 2006
 * 
 * This file is part of xtnd-commons.
 *
 * Copyright © 2006 Johan Cwiklinski
 *
 * File :               HsqldbErrors.java
 * Author's email :     johan@x-tnd.be
 * Author's Website :   http://www.x-tnd.be/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package be.xtnd.commons.db.hsql;

import java.text.MessageFormat;

import javax.swing.JOptionPane;

import be.xtnd.commons.Config;
import be.xtnd.commons.MessagesCommons;
import be.xtnd.commons.db.SqlErrors;
import be.xtnd.commons.gui.MainGui;

/**
 * Affiche un message sur certaines erreurs 
 * du moteur de base de données MySQL version 5
 * @author Johan Cwiklinski
 * @version 1.0
 */
public class HsqldbErrors extends SqlErrors{
	/** Valeur de l'erreur base de données inconnue */
	public static final int UNKNOWN_DATABASE = -94;
	/** Valeur de l'erreur accès refusé */
	public static final int ACCESS_DENIED = -37;
	/** Valeur de l'erreur base de données en utilisation */
	public static final int LAUNCHED_ONCE = -1;
	/** Valeur de l'erreur en doublon */
	public static final int DUPLICATE = -104;
	/** Valeur de l'erreur violation de clé étrangère*/
	public static final int FK_FAILS = -8;
	/** Valeur de l'erreur table existante */
	public static final int TABLE_EXISTS = -27;

	/**
	 * 
	 *
	 */
	public HsqldbErrors(){
		super(
				0,
				UNKNOWN_DATABASE,
				ACCESS_DENIED,
				LAUNCHED_ONCE,
				DUPLICATE,
				FK_FAILS,
				TABLE_EXISTS);
	}
	
	/**
	 * 
	 * @param error
	 * @param par
	 * @param conf
	 */
	public static void showError(int error, MainGui par, Config conf){
		parent = par;
		if (error == LAUNCHED_ONCE){// déjà lancée
			JOptionPane.showMessageDialog(MainGui.desktop, 
					MessagesCommons.getString("appli.isloaded"), 
					MessagesCommons.getString("appli.isloaded.title"), 
					JOptionPane.OK_OPTION + JOptionPane.ERROR_MESSAGE);
			System.exit(0);
		}
		if(error == UNKNOWN_DATABASE){//Unknown DB
			Object[] args = {conf.getSql_db(), conf.getServer_name()};
			int response = JOptionPane.showConfirmDialog(
					parent,
					MessageFormat.format(MessagesCommons.getString("database.hsqldb.errors." + UNKNOWN_DATABASE),args),
					MessagesCommons.getString("database.hsqldb.errors." + UNKNOWN_DATABASE + ".title"),
					JOptionPane.ERROR_MESSAGE+JOptionPane.YES_NO_OPTION);		
			if(response==JOptionPane.YES_OPTION){
				conf.new ConfigWindow(par);
			}else{
				cannotContinue();
			}
		}
		if(error == ACCESS_DENIED){//AccessDenied
			Object[] args = {conf.getServer_name(), conf.getSql_db(), conf.getSql_user()};
			int response = JOptionPane.showConfirmDialog(
					parent,
					MessageFormat.format(MessagesCommons.getString("database.hsqldb.errors." + ACCESS_DENIED),args),
					MessagesCommons.getString("database.hsqldb.errors." + ACCESS_DENIED + ".title"),
					JOptionPane.ERROR_MESSAGE+JOptionPane.YES_NO_OPTION);		
			if(response==JOptionPane.YES_OPTION){
				conf.new ConfigWindow(par);
			}else{
				cannotContinue();
			}			
		}
		if(error == TABLE_EXISTS){//table already exists in database
			//TODO afficher les messages dans ce cas
		}
	}
	
    private static void cannotContinue(){
		JOptionPane.showMessageDialog(
				parent,
				MessagesCommons.getString("appli.fatal"),
				MessagesCommons.getString("appli.fatal.title"),
				JOptionPane.ERROR_MESSAGE+JOptionPane.OK_OPTION);
		System.exit(0);    	
    }
}
