/*
 * MysqlErrors.java, 25 mai 2006
 * 
 * This file is part of xtnd-commons.
 *
 * Copyright © 2006 Johan Cwiklinski
 *
 * File :               MysqlErrors.java
 * Author's email :     johan@x-tnd.be
 * Author's Website :   http://www.x-tnd.be/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package be.xtnd.commons.db.mysql;

import java.text.MessageFormat;

import javax.swing.JOptionPane;

import be.xtnd.commons.Config;
import be.xtnd.commons.MessagesCommons;
import be.xtnd.commons.db.SqlErrors;
import be.xtnd.commons.gui.MainGui;

/**
 * Affiche un message sur certaines erreurs 
 * du moteur de base de données MySQL version 5
 * @author Johan Cwiklinski
 * @version 1.0
 */
public class MysqlErrors extends SqlErrors{
	/** Valeur de l'erreur hôte inconnu */
	public static final int UNKNOWN_HOST = -10;
	/** Valeur de l'erreur base de données inconnue */
	public static final int UNKNOWN_DATABASE = 1049;
	/** Valeur de l'erreur accès refusé */
	public static final int ACCESS_DENIED = 1045;
	/** Valeur de l'erreur en doublon */
	public static final int DUPLICATE = 1062;
	/** Valeur de l'erreur violation de clé étrangère*/
	public static final int FK_FAILS = 1451;
	/** Valeur de l'erreur table existante */
	public static final int TABLE_EXISTS = 1050;

	/**
	 * 
	 *
	 */
	public MysqlErrors(){
		super(
				UNKNOWN_HOST,
				UNKNOWN_DATABASE,
				ACCESS_DENIED,
				0,
				DUPLICATE,
				FK_FAILS,
				TABLE_EXISTS);

	}
	
	/**
	 * 
	 * @param error
	 * @param par
	 * @param conf
	 */
	public static void showError(int error, MainGui par, Config conf){
		parent = par;
		if(error==UNKNOWN_HOST){//Unknown Host
			Object[] args = {conf.getServer_name()};
			int response = JOptionPane.showConfirmDialog(
					parent,
					MessageFormat.format(MessagesCommons.getString("database.mysql.errors." + UNKNOWN_HOST),args),
					MessagesCommons.getString("database.mysql.errors." + UNKNOWN_HOST + ".title"),
					JOptionPane.ERROR_MESSAGE+JOptionPane.YES_NO_OPTION);		
			if(response==JOptionPane.YES_OPTION){
				conf.new ConfigWindow(par);
			}else{
				cannotContinue();
			}
		}
		if(error==UNKNOWN_DATABASE){//Unknown DB
			Object[] args = {conf.getSql_db(), conf.getServer_name()};
			int response = JOptionPane.showConfirmDialog(
					parent,
					MessageFormat.format(MessagesCommons.getString("database.mysql.errors." + UNKNOWN_DATABASE),args),
					MessagesCommons.getString("database.mysql.errors." + UNKNOWN_DATABASE + ".title"),
					JOptionPane.ERROR_MESSAGE+JOptionPane.YES_NO_OPTION);		
			if(response==JOptionPane.YES_OPTION){
				conf.new ConfigWindow(par);
			}else{
				cannotContinue();
			}
		}
		if(error==ACCESS_DENIED){//AccessDenied
			Object[] args = {conf.getServer_name(), conf.getSql_db(), conf.getSql_user()};
			int response = JOptionPane.showConfirmDialog(
					parent,
					MessageFormat.format(MessagesCommons.getString("database.mysql.errors." + ACCESS_DENIED),args),
					MessagesCommons.getString("database.mysql.errors." + ACCESS_DENIED + ".title"),
					JOptionPane.ERROR_MESSAGE+JOptionPane.YES_NO_OPTION);		
			if(response==JOptionPane.YES_OPTION){
				conf.new ConfigWindow(par);
			}else{
				cannotContinue();
			}			
		}
		if(error == TABLE_EXISTS){ //table already exists in database
			//TODO afficher les messages dans ce cas
		}
	}
	
    private static void cannotContinue(){
		JOptionPane.showMessageDialog(
				parent,
				MessagesCommons.getString("appli.fatal"),
				MessagesCommons.getString("appli.fatal.title"),
				JOptionPane.ERROR_MESSAGE+JOptionPane.OK_OPTION);
		System.exit(0);    	
    }

}
