/*
 * EscapeInternalFrame.java, 31 mai 2005
 * 
 * This file is part of xtnd-commons.
 *
 * Copyright © 2005 Johan Cwiklinski
 *
 * File :               EscapeInternalFrame.java
 * Author's email :     johan@x-tnd.be
 * Author's Website :   http://www.x-tnd.be/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package be.xtnd.commons.gui;


import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JRootPane;
import javax.swing.KeyStroke;

/**
 * Dérive JInternalFrame, implémente exit on echap 
 * ainsi que diverses méthodes communes
 *
 * @author Johan Cwiklinski
 * @version 1.0
 */
public class EscapeInternalFrame extends JInternalFrame {
	private static final long serialVersionUID = -6747591060829102159L;

	/**
	 * Constructeur par défaut
	 * Crée une <code>JInternalFrame</code> et lui attribue une îcone.
	 */
	public EscapeInternalFrame(){
		super();
		super.setFrameIcon(MainGui.imgIcon);
	}
	
	/**
	 * Construit un <code>EscapeInternalFrame</code> avec le
	 * titre passé en paramètre, définit ses propriétés d'affichage,
	 * et lui attribue une îcone.
	 * 
	 * @param title titre de la fenêtre
	 * @param resizable la fenêtre peut être retaillée ?
	 * @param closable la fenêtre peut être fermée ?
	 * @param maximizable la fenêtre peut être maximisée ?
	 * @param iconifiable la fenêtre peut être iconifiée ?
	 */
	public EscapeInternalFrame(String title, boolean resizable, boolean closable, 
            boolean maximizable, boolean iconifiable) {
		super(title, resizable, closable, maximizable, iconifiable);
		super.setFrameIcon(MainGui.imgIcon);
	}
	
	/**
	 * Crée une fenêtre popup sur click droit.
	 * 
	 * @param popup <code>JPopupMenu</code> à afficher
	 * @param text texte du menu
	 * 
	 */
	protected void popupMenu(JPopupMenu popup, String text){
		JMenu menu = new JMenu(text);
		popup.add(menu);
	}
	
	/**
	 * Configure les entrées du menu popup
	 * 
	 * @param popup menu popup à modifier
	 * @param text texte du menu
	 * @param action actionListener
	 * @param icon icône du menu
	 */
	protected void popupEntries(JPopupMenu popup, String text, String action, ImageIcon icon){
		JMenuItem item = new JMenuItem(text);
		item.setActionCommand(action);
		item.setIcon(icon);
		item.addActionListener(new java.awt.event.ActionListener() {
	        public void actionPerformed(ActionEvent e) {
	        	popupActions(e);
	            }
	        });
		popup.add(item);
	}	
	
	/**
	 * Configure les sous entrées du menu popup
	 * 
	 * @param popup menu popup à modifier
	 * @param menu <code>JMenu</code> parent
	 * @param text texte du menu
	 * @param action actionListener
	 * @param icon icône du menu
	 */
	protected void popupEntries(JPopupMenu popup, JMenu menu, String text, String action, ImageIcon icon){
		JMenuItem item = new JMenuItem(text);
		item.setActionCommand(action);
		item.setIcon(icon);
		item.addActionListener(new java.awt.event.ActionListener() {
	        public void actionPerformed(ActionEvent e) {
	        	popupActions(e);
	            }
	        });
		menu.add(item);
		popup.add(menu);
	}	
	
	/** Actions des popups
	 * @param event ActionEvent 
	 */
	protected void popupActions(ActionEvent event) {}

	/**
	 * Crée le conteneur principal.
	 * Cette méthode est utilisée pour que l'appui sur la touche Echap
	 * ferme la fenêtre.
	 * 
	 * @return JRootPane le panneau principal
	 */
	protected JRootPane createRootPane() {
		ActionListener actionListener = new ActionListener() {
			public void actionPerformed(ActionEvent actionEvent) {
				dispose();
			}
		};
		JRootPane rootPane = new JRootPane();
		KeyStroke stroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
		rootPane.registerKeyboardAction(actionListener, stroke, JComponent.WHEN_IN_FOCUSED_WINDOW);
		return rootPane;
	}
}

