/*
 * MainGui.java, 26 juin 2005
 * 
 * This file is part of xtnd-commons.
 *
 * Copyright © 2005 Johan Cwiklinski
 *
 * File :               MainGui.java
 * Author's email :     johan@x-tnd.be
 * Author's Website :   http://www.x-tnd.be/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package be.xtnd.commons.gui;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyVetoException;
import java.util.ResourceBundle;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import javax.swing.WindowConstants;

import be.xtnd.commons.Config;
import be.xtnd.commons.MDIDesktopPane;
import be.xtnd.commons.MessagesCommons;
import be.xtnd.commons.Splash;
import be.xtnd.commons.WindowMenu;
import be.xtnd.commons.db.Database;


/**
 * Fenêtre principale de l'application
 * 
 * @author Johan Cwiklinski
 * @version 1.1
 */
public abstract class MainGui extends JFrame {
	private static final long serialVersionUID = 1L;
	/** Icône de l'application, au format Image */
	public static Image icon;
	/** Icône de l'application, au format ImageIcon */
	public static ImageIcon imgIcon;
	/** Image de fermeture */
	public static ImageIcon closeImage = new ImageIcon(ClassLoader.getSystemResource("be/xtnd/icons/close.png")); //fermer
	/** Objet desktop MDI d el'application */
	public static MDIDesktopPane desktop;	
	private Dimension appliSize = new Dimension(800, 600);
    private static JScrollPane scrollPane;
    /**
     * Connexion à la base de données 
     * @see Database 
     */
    public static Database db;
    /** 
     * Configuration de l'application, chargée au lancmeent
     * @see Config 
     */
    public static Config conf;
    /** 
     * SplashScreen de l'aplication 
     * @see Splash 
     */
    public static Splash s = new Splash();
    /** Barre de statut */
    public static JLabel statusBar;
    private Image fond;
    private boolean fondEnabled;
	private static WaitFrame glassPane;

    /**
     * Crée une <code>JFrame</code>
     * 
     * @param title le titre de la frame
     * @param icon l'icone de l'application
     * @param rootMenuBar la barre de menus de l'application
     * @param toolBar la barre d'outils de l'application
     * @param config le chemin du fichier de configuration
     * @param dim la dimension de la fenêtre
     * @param bundle ResourceBundle à lire
     */
	public MainGui(String title, ImageIcon icon, JMenuBar rootMenuBar, JToolBar toolBar, 
			String config, Dimension dim, ResourceBundle bundle) {
		super(title);
		this.appliSize = dim;
		initWindow(icon, rootMenuBar, toolBar, config, bundle);
	}
	
	/**
     * Crée une <code>JFrame</code>
	 * 
	 * @param title le titre de la fenêtre
	 * @param icon l'icône de l'application
	 * @param rootMenuBar la barre de menus de l'application
	 * @param toolBar la barre d'outils de l'application
	 * @param config le chemin du fichier de configuration
	 * @param bundle ResourceBundle à lire
	 */
	public MainGui(String title, ImageIcon icon, JMenuBar rootMenuBar, JToolBar toolBar, 
			String config, ResourceBundle bundle) {
		super(title);
		initWindow(icon, rootMenuBar, toolBar, config, bundle);		
	}

	/**
     * Crée une <code>JFrame</code>
	 * 
	 * @param title le titre de la fenêtre
	 * @param icon l'icône de l'application
	 * @param rootMenuBar la barre de menus de l'application
	 * @param toolBar la barre d'outils de l'application
	 * @param config le chemin du fichier de configuration
	 * @param fond l'image de fond à afficher
	 * @param bundle ResourceBundle à lire
	 */
	public MainGui(String title, ImageIcon icon, JMenuBar rootMenuBar, JToolBar toolBar, 
			String config, Image fond, ResourceBundle bundle) {
		super(title);
		this.fondEnabled = true;
		this.fond = fond;
		initWindow(icon, rootMenuBar, toolBar, config, bundle);
	}

	/**
	 * Initialise la fenêtre et charge la configuration
	 * 
	 * @param icon icône de l'application
	 * @param rootMenuBar basrre de menus
	 * @param toolBar barre d'outils
	 * @param config fichier de configuration
	 * @param bundle ResourceBundle à lire
	 * 
	 * @see Config
	 */
	private void initWindow(ImageIcon icon, JMenuBar rootMenuBar, JToolBar toolBar, String config,
			ResourceBundle bundle){
		s = new Splash(10, bundle);
		glassPane = new WaitFrame(MessagesCommons.getString("window.messages.loading"));

		s.start();
		this.setSize(appliSize);
		this.setLocationRelativeTo(this.getParent());
		this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		
		desktop = (this.fondEnabled)?new MDIDesktopPane(fond):new MDIDesktopPane();
		scrollPane = new JScrollPane();
		MainGui.imgIcon = icon;
		MainGui.icon = icon.getImage();
		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) { 
				int response = JOptionPane.showConfirmDialog(
						desktop, 
						MessagesCommons.getString("window.messages.quit"),
						MessagesCommons.getString("window.messages.quit.title"),
						JOptionPane.YES_NO_OPTION,
						JOptionPane.QUESTION_MESSAGE);
	    		if (response == JOptionPane.YES_OPTION) {
	    			try{new Database().disconnect();}catch(Exception ex){/*au cas où...*/}
	    			System.exit(0);
	    		}
			}
		    });
		Container contentPane = this.getContentPane();
		contentPane.setLayout(new BorderLayout());
		
		int menuPosition = 0;
		if(rootMenuBar.getComponentCount()!=0) menuPosition = rootMenuBar.getComponentCount()-1; 
		rootMenuBar.add(new WindowMenu(desktop),menuPosition);
		this.setJMenuBar(rootMenuBar);
	    this.setIconImage(MainGui.icon);

		if(toolBar!=null)
			contentPane.add(toolBar, BorderLayout.NORTH);

		scrollPane.getViewport().add(desktop);
    	contentPane.add(scrollPane,BorderLayout.CENTER);
    	contentPane.add(status(), BorderLayout.SOUTH);
		desktop.getRootPane().setGlassPane(glassPane);

		conf = new Config(config);
		conf.loadConfig(false);

		Object c = new Object();
		while(conf.loaded==false){
			try{
				s.end();
				synchronized(c) { c.wait(500); }
				System.out.print(".");
			} catch(InterruptedException ex) { }			
		}

		Object o=new Object();
		System.out.print(MessagesCommons.getString("window.messages.loading"));
		while(s.ended==false){
			try{
				synchronized(o) { o.wait(500); }
				System.out.print(".");
			} catch(InterruptedException ex) { }
		}
		System.out.println(".");
		o=null;
		this.setVisible(true);
		verifUnicite(Config.name,null);
		s.end();		
		try {loadDb();} catch (Exception e1) {}
	}	
	
	/**
	 * Crée un label pour l'affichage de statuts divers 
	 * 
	 * @return JLabel label de statut
	 */
	private JLabel status(){
		 statusBar = new JLabel();
		 statusBar.setPreferredSize(new Dimension(200,20));
		 return statusBar;
	}
    
    /**
     * Vérifie l'unicité de la fenêtre basée sur son nom
     * 
     * @param name nom textuel de la fenêtre
     * @param defaultLoc emplacement de la fenêtre
     * 
     * @return true si la fenêtre existe, false sinon
     */
	public static boolean verifUnicite(String name, Point defaultLoc){
		JInternalFrame[] frames = desktop.getAllFrames();
		boolean load = true;
		int i = 0;
		while(i < frames.length){
			if(frames[i].getName().equals(name)){
				System.out.println("name matches : "+name);
				load = false;
				frames[i].toFront();
				if(defaultLoc != null)
					frames[i].setLocation(defaultLoc);
				try {
					frames[i].setSelected(true);
				} catch (PropertyVetoException e) {}
			}
			i++;
		}
		return load;
	}

	/**
	 * Centre une fenêtre fille sur le bureau.
	 * 
	 * @param dim
	 * @return Point emplacement de la fenêtre
	 */
	public static Point centerOnDesktop(Dimension dim){
		double w = (desktop.getWidth() - dim.getWidth())/2;
		double h = (desktop.getHeight() - dim.getHeight())/2;
		if(w<0) w = 20;
		if(h<0) h = 20;
		return new Point((int)w, (int)h);
	}

	/**
	 * (Dés)Active l'écran d'attente avec le message par défaut
	 * @param start true pour activer l'écran, false pour le désactiver
	 */
	public static void attente(boolean start){
		attente(start, "-1");
	}
	
	/**
	 * (Dés)Active l'écran d'attente
	 * @param start true pour activer l'écran, false pour le désactiver
	 * @param msg message à afficher ou '-1' pour le message par défaut
	 */
	public static void attente(boolean start, String msg){
		if(!msg.equals("-1")) glassPane.setText(msg);
		if(start)
			glassPane.start();
		if(!start)
			glassPane.stop();
	}

	/**
	 * Chargement de la base de données
	 * @throws Exception 
	 */
	public abstract void loadDb() throws Exception;
}
