/*
 * Cryptographie.java, 29 août 2005
 * 
 * This file is part of xtnd-commons.
 *
 * Copyright © 2005 Johan Cwiklinski
 *
 * File :               Cryptographie.java
 * Author's email :     johan@x-tnd.be
 * Author's Website :   http://www.x-tnd.be/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package be.xtnd.commons.security;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyVetoException;
import java.util.Random;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JOptionPane;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.WindowConstants;

import be.xtnd.commons.MessagesCommons;
import be.xtnd.commons.gui.EscapeInternalFrame;
import be.xtnd.commons.gui.MainGui;

import com.jeta.forms.components.border.TitledBorderLabel;
import com.jeta.forms.components.panel.FormPanel;
import com.jeta.forms.components.separator.TitledSeparator;
import com.jgoodies.looks.plastic.PlasticInternalFrameUI;
import com.toedter.components.JSpinField;

/**
 * Crypte et/ou décrypte une chaîne de caractères passée
 * en paramètre. 
 * Permet également la création aléatoire d'un mot de passe.
 * La classe CryptoWindow fournit une interface graphique basique
 *
 * @author Thierry Selva
 * @author Johan Cwiklinski
 * @version 1.0
 */
public class Cryptographie {
    private static int base=47,intervalle=109-base;
	
    private static int modifie_intervalle(int n,int sens) {
	    if (sens==-1) {
	        if (n>96) return n-13;
	        else if (n>64) return n-7;
	        else return n;
	    } else {
	        if (n>83) return n+13;
	        else if (n>57) return n+7;
	        else return n;
	    }
	}
    
    /**
     * Génère une chaîne aléatoire de longueur spécifiée
     * 
     * @param n longueur de la chaîne désirée
     * @return chaîne aléatoire
     */
    public static String aleatoire(int n) {
        Random generateur = new Random(System.currentTimeMillis());
        int r;
        StringBuffer chaine=new StringBuffer(n);
        for (int i=0;i<n;i++) {
            r=generateur.nextInt(intervalle);
            chaine.append((char) modifie_intervalle(r+base+1,1));
        }
        return chaine.toString();
    }
    
    private static String cryptage(String password,int position) {
	    int longueur=password.length()/*,m*/;
	    StringBuffer crypte=new StringBuffer(longueur);
	    int init=modifie_intervalle((int) password.charAt(position),-1)-base;
	    //char c;
	    for (int i=0;i<longueur;i++) {
	        if (i==position) crypte.append(password.charAt(position));
	        else crypte.append((char) modifie_intervalle((modifie_intervalle((int) password.charAt(i),-1)+init-(base+1))%intervalle+(base+1),1));
	        init=modifie_intervalle((int) password.charAt(i),-1)-base;
	    }
	    return crypte.toString();
	}
	
	private static String decryptage(String crypte,int position) {
	    int base=47,intervalle=109-base;
	    int longueur=crypte.length();
	    StringBuffer password=new StringBuffer(longueur);
	    int n,init=modifie_intervalle((int) crypte.charAt(position),-1)-base;
	    //char c;
	    for (int i=0;i<longueur;i++) {
	        if (i==position) password.append(crypte.charAt(position));
	        else {
	            n=modifie_intervalle((int) crypte.charAt(i),-1)-init-base;
	            n=(n<1)?n+=intervalle:n;
	            password.append((char) modifie_intervalle(n+base,1));
	        }
	        init=modifie_intervalle((int) password.charAt(i),-1)-base;
	    }
	    return password.toString();
	}
	
	/**
	 * Crypte une chaîne
	 * 
	 * @param chaine chaîne à crypter
	 * @return chaîne cryptée
	 */
	public static String crypte(String chaine) {
	    if(chaine.equalsIgnoreCase("")){
	    	return null;
	    }else{
	    	return cryptage(cryptage(chaine,chaine.length()-2),2);	    	
	    }
	}
	
	/**
	 * Décrypte une chaîne
	 * 
	 * @param chaine chaîne à décrypter
	 * @return chaîne décryptée
	 */
	public static String decrypte(String chaine) {
	    if(chaine.equalsIgnoreCase("")){
	    	return null;
	    }else{
	    	return decryptage(decryptage(chaine,2),chaine.length()-2);
	    }
	}

	/**
	 * Interface utilisateur pour l'utilisation des fonctions de cryptage 
	 * 
	 * @author Johan Cwiklinski
	 * @author Thierry Selva
	 */
	public class CryptoWindow extends EscapeInternalFrame{
		private static final long serialVersionUID = 1030755448571865099L;
		private JButton okay, cancel;
		private TitledSeparator title;
		private TitledBorderLabel mode;
		private JLabel param_label, result_label;
		private JTextField param, result;
		private JRadioButton crypt, decrypt;
		private JCheckBox aleat;
		private JSpinField nbre_aleat;
		
		/**
		 * Constructeur par défaut.
		 * Crée la fenêtre et initialise les composants.
		 * Il s'agit ici d'une <code>JInternalFrame</code>, il
		 * est donc nécéssaire d'implémenter auparavant une <code>JFrame</code>
		 * et un <code>JDesktopPane</code> pour l'accueilir.
		 *
		 */
		public CryptoWindow(){
    		super();
    		super.setTitle(MessagesCommons.getString("window.crypto.title"));
            putClientProperty(
                    PlasticInternalFrameUI.IS_PALETTE,
                    Boolean.TRUE);
    		FormPanel pane = new FormPanel( "be/xtnd/commons/gui/descriptions/cryptform.jfrm" );

    		initLabels(pane);
    		
    		title = (TitledSeparator)pane.getComponentByName("title");
    		title.setText(MessagesCommons.getString("window.crypto.title"));
    		
    		aleat = pane.getCheckBox("aleatoire");
    		aleat.addActionListener(new AleatEvent());
    		aleat.setText(MessagesCommons.getString("window.crypto.labels.random"));
    		aleat.setToolTipText(MessagesCommons.getString("window.crypto.labels.random.tooltip"));
    		nbre_aleat = (JSpinField)pane.getComponentByName("nbre_aleat");
    		
    		crypt = pane.getRadioButton("crypt");
    		crypt.setText(MessagesCommons.getString("window.crypto.labels.crypt"));
    		crypt.setToolTipText(MessagesCommons.getString("window.crypto.labels.crypt.tooltip"));
    		
    		decrypt = pane.getRadioButton("decrypt");
    		decrypt.setText(MessagesCommons.getString("window.crypto.labels.decrypt"));
    		decrypt.setToolTipText(MessagesCommons.getString("window.crypto.labels.decrypt.tooltip"));
    		
    		param = pane.getTextField("param");
    		result = pane.getTextField("result");
    		
    		mode = (TitledBorderLabel)pane.getComponentByName("mode");
    		mode.setText(MessagesCommons.getString("window.crypto.title_mode"));
    		
    		okay = (JButton)pane.getButton("okay");
    		okay.setText(MessagesCommons.getString("window.crypto.buttons.okay"));
    		okay.addActionListener(new OkayEvent());
    		
    		cancel = (JButton)pane.getButton("annuler");
    		cancel.setText(MessagesCommons.getString("window.crypto.buttons.cancel"));
    		cancel.addActionListener(new CancelEvent());
    		
    		add(pane);

    		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
    		setClosable(true);
    		
        	pack();
        	setVisible(true);
        	setName("cryptowindow");
        	getRootPane().setDefaultButton(okay);
    		MainGui.desktop.add(this,JLayeredPane.MODAL_LAYER);
        	setLocation(MainGui.centerOnDesktop(getSize()));
        	try {
    			setSelected(true);
    		} catch (PropertyVetoException e) {}			

		}
				
		private void initLabels(FormPanel pane){
			param_label = pane.getLabel("param_label");
			param_label.setText(MessagesCommons.getString("window.crypto.labels.param"));
			
			result_label = pane.getLabel("result_label");
			result_label.setText(MessagesCommons.getString("window.crypto.labels.result"));
		}
		
		class OkayEvent implements ActionListener{
			/** Action du click sur Okay 
			 * @param e 
			 */
			public void actionPerformed(ActionEvent e) {
				String parametre;
				if(param.getText().equals("") && !aleat.isSelected()) {
	    			JOptionPane.showMessageDialog(
	    					MainGui.desktop, 
							MessagesCommons.getString("window.crypto.messages.noparam"),
							MessagesCommons.getString("window.crypto.messages.noparam.title"),
							JOptionPane.OK_OPTION+JOptionPane.ERROR_MESSAGE);
				}else{
					parametre = (aleat.isSelected())?aleatoire(nbre_aleat.getValue()):param.getText();
					if(aleat.isSelected()) param.setText(parametre);
					result.setText((crypt.isSelected())?crypte(parametre):decrypte(parametre));
					result.setText((decrypt.isSelected())?decrypte(parametre):crypte(parametre));
				}
			}		
		}
		class CancelEvent implements ActionListener{
			/** Action du click sur annuler 
			 * @param e ActionEvent 
			 */
			public void actionPerformed(ActionEvent e) {
				dispose();
			}		
		}

		class AleatEvent implements ActionListener{
			/** Action du click sur aléatoire 
			 * @param e ActionEvent
			 */
			public void actionPerformed(ActionEvent e) {
				nbre_aleat.setEnabled(aleat.isSelected());
			}		
		}
	}
}
