/*
 * MailFieldVerifier.java, 21 sept. 2005
 * 
 * This file is part of xtnd-commons.
 *
 * Copyright © 2005 Johan Cwiklinski
 *
 * File :               MailFieldVerifier.java
 * Author's email :     johan@x-tnd.be
 * Author's Website :   http://www.x-tnd.be/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package be.xtnd.validators;

import javax.swing.InputVerifier;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import org.apache.commons.validator.EmailValidator;

import be.xtnd.commons.MessagesCommons;
import be.xtnd.commons.gui.MainGui;


/**
 * Vérifie la validité d'une adresse email.
 * Basé sur l'API commons validator apache.
 *
 * @author Johan Cwiklinski
 * @version 1.0
 */
public class MailFieldVerifier extends InputVerifier{
	private String mail_value;
	
	/**
	 * Constructeur par défaut
	 * 
	 * @param mail adresse mail à vérifier
	 * @see InputVerifier
	 */
	public MailFieldVerifier(String mail) {
		this.mail_value = mail;
	}

	/**
	 * Renvoie le résultat de la vérification de l'adresse
	 * 
	 * @param input composant en entrée
	 * @return true si l'adresse est valide, false sinon
	 * @see EmailValidator
	 */
	@Override
	public boolean verify(JComponent input) {
		 if (input instanceof JTextField) {
		 	EmailValidator val = EmailValidator.getInstance();
		 	JTextField field = (JTextField)input;
		 	if(!val.isValid(field.getText()) && !field.getText().equals("")){
          		int response = JOptionPane.showConfirmDialog(
          				MainGui.desktop, 
						MessagesCommons.getString("window.messages.error.mail"),
						MessagesCommons.getString("window.messages.error"),
						JOptionPane.YES_NO_OPTION,
						JOptionPane.WARNING_MESSAGE);
          		if(response==JOptionPane.NO_OPTION){
         			field.setText(mail_value);
          			return true;
          		}
			 	return val.isValid(field.getText());
		 	}
		 }
		 return true;
	}
	
	/**
	 * Le composant doit-il céder le focus.
	 * 
	 * @param input composant en entrée
	 * @return true si l'adresse mail est valide, false sinon
	 */
	@Override
    public boolean shouldYieldFocus(JComponent input) {
          return verify(input);
    }	
}
